#!/usr/bin/env python
# encoding=utf-8
# maintainer: rgaudin

import weakref


class Dependency(object):

    __instances__ = []

    DEBIAN_PACKAGE = 0
    PIP_PACKAGE = 1
    PIP_URL = 2
    GIT_REPO = 3
    KINDS = ((DEBIAN_PACKAGE, 'Debian package(s)'),
             (PIP_PACKAGE, 'PIP package(s)'),
             (PIP_URL, 'PIP URL(s)'),
             (GIT_REPO, 'Git repositor(y|ies)'))

    def __init__(self, source=None, branch=None, revision=None, \
                 lib_path=None, lib_name=None, pip_file=None, \
                 clone_name=None, run_tests=False, symlink=True, \
                 install=False, kind=GIT_REPO, *args, **kwargs):

        # keep track of instances
        self.__instances__.append(weakref.ref(self))

        self.source = source
        self.branch = branch
        self.revision = revision
        self.lib_path = lib_path
        self.lib_name = lib_name
        self.pip_file = pip_file
        self.clone_name = clone_name
        self.run_tests = run_tests
        self.symlink = symlink
        self.install = install
        self.kind = kind

    # source/url
    def set_source(self, value):
        self._source = value

    def get_source(self):
        return unicode(self._source)
    source = property(get_source, set_source)

    # branch
    def set_branch(self, value):
        self._branch = value

    def get_branch(self):
        return self._branch
    branch = property(get_branch, set_branch)

    # revision
    def set_revision(self, value):
        self._revision = value

    def get_revision(self):
        return self._revision
    revision = property(get_revision, set_revision)

    # lib_path
    def set_lib_path(self, value):
        self._lib_path = value

    def get_lib_path(self):
        return self._lib_path
    lib_path = property(get_lib_path, set_lib_path)

    # lib_name
    def set_lib_name(self, value):
        self._lib_name = value

    def get_lib_name(self):
        return self._lib_name
    lib_name = property(get_lib_name, set_lib_name)

    # pip_file
    def set_pip_file(self, value):
        self._pip_file = value

    def get_pip_file(self):
        return self._pip_file
    pip_file = property(get_pip_file, set_pip_file)

    # clone_name
    def set_clone_name(self, value):
        self._clone_name = value

    def get_clone_name(self):
        return self._clone_name or self._get_name_from_git()
    clone_name = property(get_clone_name, set_clone_name)

    # run_tests
    def set_run_tests(self, value):
        self._run_tests = bool(value)

    def get_run_tests(self):
        return bool(self._run_tests)
    run_tests = property(get_run_tests, set_run_tests)

    # symlink
    def set_symlink(self, value):
        self._symlink = bool(value)

    def get_symlink(self):
        return bool(self._symlink)
    symlink = property(get_symlink, set_symlink)

    # install
    def set_install(self, value):
        self._install = bool(value)

    def get_install(self):
        return bool(self._install)
    install = property(get_install, set_install)

    # kind
    def set_kind(self, value):
        if not [kind[0] for kind in self.KINDS].count(value):
            raise ValueError
        self._kind = value

    def get_kind(self):
        return self._kind
    kind = property(get_kind, set_kind)

    def _get_name_from_git(self):
        h, s, t = self.source.rpartition('/')
        return t.rsplit('.git')[0]

    def is_git(self):
        return self.kind in (self.GIT_REPO,)

    def is_pip_package(self):
        return self.kind == self.PIP_PACKAGE

    def is_pip_url(self):
        return self.kind == self.PIP_URL

    def is_pip(self):
        return self.kind in (self.PIP_PACKAGE, self.PIP_URL)

    def is_deb(self):
        return self.kind == self.DEBIAN_PACKAGE

    def is_package(self):
        return self.kind in (self.PIP_PACKAGE, self.DEBIAN_PACKAGE)

    @classmethod
    def canonical_name_from(cls, url=''):
        xpart = url.rpartition('/')[2].rpartition('.')
        index = 0 if xpart[0] else 2
        return xpart[index]

    def github_public(self):
        return self.source.replace('git@github.com:', 'git://github.com/')

    def github_private(self):
        return self.source.replace('git://github.com/', 'git@github.com:')

    @property
    def module_name(self):
        if self.is_package():
            return self.source
        if self.lib_name:
            return self.lib_name
        return Dependency.canonical_name_from(url=self.source)

    @classmethod
    def instances(cls):
        return [inst.__call__() for inst in cls.__instances__]

    @classmethod
    def all_by_kind(cls, kind):
        results = []
        for inst in cls.instances():
            if inst.kind == kind:
                results.append(inst)
        return results

    def __str__(self):

        return u"<%(name)s>" % {'name': self.module_name()}

    def __rst__(self):
        if self.is_deb():
            return u"**%(name)s**\n\n" \
                    "    ``sudo aptitude install %(name)s``" \
                   % {'name': self.module_name}
        if self.is_pip_package():
            return u"**%(name)s**\n\n" \
                    "    ``pip install %(name)s``" \
                   % {'name': self.module_name}
        if self.is_pip_url():
            return u"**%(name)s**\n\n" \
                    "    ``pip install -e %(source)s``" \
                   % {'name': self.module_name, 'source': self.source}
        if self.is_git():
            rst = u"**%(name)s**\n\n" \
                    "    ``git clone %(source)s``" \
                   % {'name': self.module_name, 'source': self.source}
            if self.branch:
                rst += u"\n    ``git checkout %(branch)s``" \
                       % {'branch': self.branch}
            if self.revision:
                rst += u"\n    ``git reset --hard %(revision)s``" \
                       % {'revision': self.revision}
            return rst

    @classmethod
    def kind_name(cls, kind):
        for kind_id, kind_name in cls.KINDS:
            if kind == kind_id:
                return kind_name
        return None

    @classmethod
    def kind_rst(cls, kind):
        kind_name = cls.kind_name(kind)
        line = u"".zfill(kind_name.__len__()).replace('0', '~')
        return u"%(name)s\n%(line)s" % {'name': kind_name, 'line': line}

    @classmethod
    def __rst_all__(cls):

        rst = u""
        # store dependencies in a hash indexed by type
        deps = {}
        for kind, kind_name in Dependency.KINDS:
            deps[kind] = Dependency.all_by_kind(kind)

        for kind, kind_deps in deps.items():
            if kind_deps:
                deps_rst = u"- " + u"\n\n- ".join([k.__rst__() \
                                                for k in kind_deps])
                rst += "%(kind)s\n\n%(deps)s\n\n" \
                       % {'kind': cls.kind_rst(kind), \
                          'deps': deps_rst}
        return rst



#!/usr/bin/env python
# encoding=utf-8
# maintainer: rgaudin

import os

from .constants import *
from .raw_tools import *
from fabric.colors import blue, red


def install(dep, working_dir=DEFAULT_WD):

    print('--------\n' + blue('Installing %s' % dep) + '\n--------')
    if dep.kind in (dep.PIP_URL, dep.PIP_PACKAGE):
        if dep.kind == dep.PIP_URL:
            pip_install_url(dep.source)
        elif dep.kind == dep.PIP_PACKAGE:
            pip_install_package(dep.source)
        return JOB_DONE

    if dep.kind == dep.DEBIAN_PACKAGE:
        deb_install_package(package=dep.source)
        return JOB_DONE

    with cd(working_dir):
        # clone or pull
        if os.path.exists(dep.clone_name):
            with cd(dep.clone_name):
                update(dep)
        else:
            clone(dep, working_dir)

        # now we'll be working on the new folder
        with cd(dep.clone_name):
            # switch branch
            switch_branch(dep)

            # get revision
            get_revision(dep)

            # install
            setup_install(dep)

            # symlink site packages
            create_symlink(dep)

            # install pip-req
            install_pip_req(dep)


def syncdb_only(rep, working_dir=DEFAULT_WD):

    # ./manage.py syncdb --noinput
    django_syncdb(at=working_dir)


def syncdb(rep, working_dir=DEFAULT_WD, \
           username=None, password=None, email=None):

    # ./manage.py syncdb --noinput
    django_syncdb(at=working_dir)
    # ./manage.py createsuperuser
    django_create_superuser(at=working_dir, \
                       username=username, password=password, email=email)


def migrate(rep, working_dir=DEFAULT_WD):

    # ./manage migrate (for South)
    django_migrate(at=working_dir)


def loadfixtures(rep, fixtures='initial_data.json', working_dir=DEFAULT_WD):

    # ./manage.py loaddata
    django_loaddata(at=working_dir, fixtures=fixtures)


def clone(dep, working_dir=DEFAULT_WD):

    # git clone the repository
    git_clone(url=dep.source, at=working_dir, to=dep.clone_name)


def update(dep, working_dir=DEFAULT_WD):

    # git pull the repository
    git_pull(at=working_dir)


def switch_branch(dep, working_dir=DEFAULT_WD):

    if not dep.branch:
        return NOTHING_TO_DO

    with cd(working_dir):
        if not git_branch_exists(dep.branch, at=working_dir):
            switch_remote_branch(branch=dep.branch, local_branch=dep.branch, \
                                 at=working_dir)
        else:
            switch_local_branch(branch=dep.branch, at=working_dir)


def get_revision(dep, working_dir=DEFAULT_WD):

    if not dep.revision:
        return NOTHING_TO_DO

    git_get_revision(revision=dep.revision, at=working_dir)


def create_symlink(dep, working_dir=DEFAULT_WD):

    if not dep.symlink:
        return NOTHING_TO_DO

    # create lib symlink on site-packages
    symlink_lib(lib_name=dep.lib_name, to=working_dir, lib_path=dep.lib_path)


def setup_install(dep, working_dir=DEFAULT_WD):

    if not dep.install:
        return NOTHING_TO_DO

    python_install(at=working_dir)


def install_pip_req(dep, working_dir=DEFAULT_WD):

    if not dep.pip_file:
        return NOTHING_TO_DO

    req_file = absolute_path(dep.pip_file, at=working_dir)
    pip_install_req(req_file)


def github_to_private(dep, working_dir=DEFAULT_WD):

    if not dep.is_git():
        return NOTHING_TO_DO

    import datetime

    with settings(warn_only=True):
        temp_clone = git_clone(url=dep.github_private(), at='/tmp', \
                               to=datetime.datetime.now().strftime('rep_%s'))

    if not temp_clone.failed:
        with cd(dep.clone_name):
            # git config to change remote origin url
            git_change_config(name='remote.origin.url', \
                          value=dep.github_private(), at=working_dir)



#!/usr/bin/env python
# encoding=utf-8

'''
    Example fab file for use with ylmfab.

    Usage:
    $ fab -f demo.py deploy
'''

import os

from fabric.api import *
import ylmfab

# dependencies as Dependency objects

bolibana = ylmfab.Dependency()
bolibana.url = 'git://github.com/bolibana/bolibana.git'
bolibana.branch = 'who'
bolibana.lib_path = os.path.join('lib', 'rapidsms')
bolibana.lib_name = 'rapidsms'
bolibana.pip_file = 'pip-requires.txt'
bolibana.revision = 'ac43a836e0de060de2af4dae259f4db6a387999c'

direct_sms = ylmfab.Dependency()
direct_sms.url = 'git://github.com/rgaudin/Direct-SMS.git'
direct_sms.branch = 'new-core'
direct_sms.lib_name = 'direct_sms'

simple_locations = ylmfab.Dependency()
simple_locations.url = 'git://github.com/yeleman/simple_locations.git'

django_eav = ylmfab.Dependency()
django_eav.url = 'git+git://github.com/mvpdev/django-eav.git#egg=django-eav'
django_eav.kind = ylmfab.Dependency.PIP_URL

dependencies = [bolibana, direct_sms, simple_locations, django_eav]

# fabric methods/optioms


def deploy():

    # install dependencies
    for dependency in dependencies:
        ylmfab.install(dependency)



#!/usr/bin/env python
# encoding=utf-8
# maintainer: rgaudin

import setuptools

setuptools.setup(
    name='ylmfab',
    version=__import__('ylmfab').__version__,
    license='GNU Lesser General Public License (LGPL), Version 3',

    install_requires=['Fabric>=0.9.2'],
    provides=['ylmfab'],

    description='fabfile helper.',
    long_description=open('README.rst').read(),

    url='http://github.com/yeleman/ylmfab',

    packages=['ylmfab'],

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or '
        'Lesser General Public License (LGPL)',
        'Programming Language :: Python',
    ],
)



#!/usr/bin/env python
# encoding=utf-8
# maintainer: rgaudin

import sys
import os

from fabric.api import local, cd, settings, hide

from .constants import *


def python_version():
    return  '.'.join(str(p) for p in sys.version_info[:2])


def site_packages_path():
    return '%(prefix)s/lib/python%(version)s/site-packages' \
            % {'prefix': path_prefix(), 'version': python_version()}


def path_prefix():
    if is_virtual():
        return os.environ['VIRTUAL_ENV']
    else:
        return '%s/local/' % sys.prefix


def is_virtual():
    return 'VIRTUAL_ENV' in os.environ


def absolute_path(path, at=DEFAULT_WD):

    with cd(at):
        root = get_pwd()
    return os.path.join(root, path)


def get_pwd():
    with settings(
        hide('warnings', 'running', 'stdout', 'stderr'), \
        warn_only=True):
        return local('pwd')


def git_clone(url, at=DEFAULT_WD, to=None):

    """ clone a git repository

    Keyword arguments:
    url -- repository url
    at -- clone's parent folder (where command will be initiated)
    to -- name of the clone's folder

    """

    path = ' %s' % to if to else ''
    with cd(at):
        return local('git clone %(url)s%(path)s' \
                     % {'url': url, 'path': path}, capture=False)


def git_pull(at=DEFAULT_WD):

    """ pulls a git repository

    Keyword arguments:
    at -- repository's folder

    """

    with cd(at):
        local('git pull', capture=False)


def switch_local_branch(branch, at=DEFAULT_WD):

    with cd(at):
        local('git checkout %(branch)s' % {'branch': branch}, capture=False)


def switch_remote_branch(branch, local_branch=None, at=DEFAULT_WD):

    if not local_branch:
        local_branch = branch

    with cd(at):
        local('git checkout -b %(local_branch)s %(branch)s' \
              % {'branch': branch, 'local_branch': local_branch}, \
              capture=False)


def git_branch_exists(branch, at=DEFAULT_WD):

    with cd(at):
        with settings(
            hide('warnings', 'running', 'stdout', 'stderr'), \
            warn_only=True):
            return bool(local('git branch -a |grep %(branch)s | wc -l' \
                              % {'branch': branch}))


def git_get_revision(revision, at=DEFAULT_WD):

    with cd(at):
        local('git reset --hard %(rev)s' % {'rev': revision}, capture=False)


def git_change_config(name, value, at=DEFAULT_WD):

    with cd(at):
        local('git config %(name)s "%(value)s"' \
              % {'name': name, 'value': value}, capture=False)


def symlink_lib(lib_name=None, to=DEFAULT_WD, lib_path=None, destination=None):

    with cd(to):
        dep_dir = get_pwd()

    if not destination:
        destination = site_packages_path()

    if lib_path:
        dep_lib = os.path.join(dep_dir, lib_path)
    else:
        dep_lib = dep_dir

    if not lib_name:
        lib_name = '.'

    lib_name = absolute_path(path=lib_name, at=destination)

    local('ln -fs %(dep_lib)s %(dep_name)s' % \
          {'dep_lib': dep_lib, 'dep_name': lib_name}, capture=False)


def python_install(at):

    # sudo the install if not on virtualenv
    launcher = local if is_virtual() else sudo

    with cd(at):
        launcher('./setup.py install', capture=False)


def pip_install_package(package):

    # sudo the install if not on virtualenv
    launcher = local if is_virtual() else sudo

    launcher('pip install %(package)s' % {'package': package}, capture=False)


def pip_install_url(url):

    # sudo the install if not on virtualenv
    launcher = local if is_virtual() else sudo

    launcher('pip install -e %(url)s' % {'url': url}, capture=False)


def pip_install_req(req_file):

    # sudo the install if not on virtualenv
    launcher = local if is_virtual() else sudo

    launcher('pip install -r %(pip_file)s' \
             % {'pip_file': req_file}, capture=False)


def deb_install_package(package):

    sudo('aptitude install %(package)s' % {'package': package}, capture=False)


def django_syncdb(at=DEFAULT_WD):

    with cd(at):
        local('./manage.py syncdb --noinput', capture=False)


def django_create_superuser(at=DEFAULT_WD, \
                       username=None, password=None, email=None, **args):

    cmd = './manage.py createsuperuser'
    if username:
        cmd = '%(cmd)s --username="%(username)s"' \
               % {'cmd': cmd, 'username': username}
    if email:
        cmd = '%(cmd)s --email="%(email)s"' % {'cmd': cmd, 'email': email}

    with cd(at):
        local(cmd, capture=False)


def django_migrate(at=DEFAULT_WD):

    with cd(at):
        local('./manage.py migrate', capture=False)


def django_loaddata(at=DEFAULT_WD, fixtures='initial_import.json'):

    with cd(at):
        local('./manage.py loaddata %(fixtures)s' % {'fixtures': fixtures}, \
              capture=False)


def dependencies_to_rst(dependencies):

    from .models import Dependency

    return Dependency.__rst_all__()



#!/usr/bin/env python
# encoding=utf-8
# maintainer: rgaudin

VERSION = (0, 1, 0)


def get_version():
    version = "%s.%s" % (VERSION[0], VERSION[1])
    if VERSION[2] != 0:
        version = "%s.%s" % (version, VERSION[2])
    return version

__version__ = get_version()

from .constants import *
from .models import Dependency
try:
    from .raw_tools import *
    from .tools import *
except ImportError:
    pass
